package cz.jzitnik.game.interactions.list;

import com.fasterxml.jackson.databind.ObjectMapper;
import cz.jzitnik.api.ApiService;
import cz.jzitnik.api.requests.InteractionRequest;
import cz.jzitnik.api.responses.InteractionResponse;
import cz.jzitnik.api.types.Character;
import cz.jzitnik.api.types.ItemType;
import cz.jzitnik.utils.Cli;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.io.Console;
import java.io.IOException;
import java.util.Arrays;

@AllArgsConstructor
public class NumberGuessingGame {
    private Character character;
    private ApiService apiService;
    private String playerKey;

    @Getter
    private static class NumberGuessingResponse {
        public enum Type {
            GAME_CREATED,
            GAME_WON,
            GAME_LOST,
            GUESS_FEEDBACK
        }

        private Type type;
        private String message;
        private int attemptsLeft;
    }

    public void play() throws IOException {
        var response = apiService.isInteracting(playerKey, character.getId()).execute();
        var interacting = response.body().getData().get();

        if (!interacting) {
            init();
        } else {
            // Resume the game where it was left off
            var body = apiService.interact(playerKey, new InteractionRequest("get_data", character.getId())).execute().body();
            var data = body.getData().get();
            gameContinue(data);
        }
    }

    private void init() throws IOException {
        var response = apiService.getMe(playerKey).execute();
        var me = response.body().getData().get();

        if (me.getInventory().stream().noneMatch(item -> item.getItemType().equals(ItemType.COAL))) {
            // Hráč nemá uhlí
            return;
        }

        var options = new String[] { "Dát kovářovi uhlí", "Nedat kovářovi uhlí" };
        var index = Cli.selectOptionIndex(Arrays.asList(options));

        if (index == 1) {
            return;
        }

        var body = apiService.interact(playerKey, new InteractionRequest("", character.getId())).execute().body();
        var data = body.getData().get();
        gameContinue(data);
    }

    private void gameContinue(InteractionResponse data) throws IOException {
        ObjectMapper objectMapper = new ObjectMapper();
        var gameData = objectMapper.readValue(data.getResponseText(), NumberGuessingResponse.class);

        switch (gameData.getType()) {
            case GAME_WON -> {
                Cli.type(character, gameData.getMessage());
                Cli.gotItems(data.getItems());
            }
            case GAME_LOST -> Cli.type(character, gameData.getMessage());
            case GUESS_FEEDBACK, GAME_CREATED -> {
                Cli.type(character, gameData.getMessage());
                System.out.println("Počet pokusů: " + gameData.getAttemptsLeft());

                var guess = askForGuess();

                var response = apiService.interact(playerKey, new InteractionRequest(guess, character.getId())).execute();

                var body = response.body().getData().get();

                gameContinue(body);
            }
        }
    }

    private String askForGuess() {
        Console console = System.console();

        String data = console.readLine("Zadejte číslo: ");

        try {
            int num = Integer.parseInt(data);

            if (num < 1 || num > 100) {
                Cli.error("Číslo musí být v rozmezí 1-100");
                return askForGuess();
            }

            return data;
        } catch (NumberFormatException e) {
            Cli.error("Neplatný vstup! Zadejte platné číslo.");
            return askForGuess();
        }
    }
}
